package uk.co.sculptor.demos.stockcontrol;

import android.accounts.OperationCanceledException;
import android.content.AsyncTaskLoader;
import android.content.Context;
import android.util.Log;

import java.io.IOException;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;

import uk.co.sculptor.kflibjava.Database;
import uk.co.sculptor.kflibjava.Field;
import uk.co.sculptor.kflibjava.FileIndex;
import uk.co.sculptor.kflibjava.KeyedFile;
import uk.co.sculptor.kflibjava.KfException;
import uk.co.sculptor.kflibjava.Server;

/**
 * Created on 03 June 2015.
 * <p/>
 * Load the remote Sculptor database stock products. For explanations on this implementation see:
 * <a href="http://www.androiddesignpatterns.com/2012/08/implementing-loaders.html">
 *     http://www.androiddesignpatterns.com/2012/08/implementing-loaders.html</a>
 *
 * @author Sculptor Ltd.
 */
public class StockLoader extends AsyncTaskLoader<StockLoader.StockLoaderResponse> {
	/*
	 * Defines the object that the StockLoader will create as a response when the loading process
	 * has finished
	 */
	public static class StockLoaderResponse {
		private final List<Product> m_stockList;
		private final Exception m_exception;

		public StockLoaderResponse(List<Product> stockList, Exception exception) {
			m_stockList = stockList;
			m_exception = exception;
		}

		public List<Product> getStockList() {
			return m_stockList;
		}

		public Exception getException() {
			return m_exception;
		}
	}

	// We save the last delivered result
	StockLoaderResponse m_stockLoaderResponse;

	/**
	 * Values that refer to the server address and keyed file path. Modify to your needs.
	 */
	private static final String SERVER_NAME = "192.168.1.105";
	private static final String STOCK_FILENAME = "c:/sc/demos/stockcontrol/data/stock";
	private static final String STOCK_LOG = "c:/sc/demos/stockcontrol/logs/androidclient.log";

	/**
	 * The class name used to tag the log messages.
	 */
	private static final String TAG = StockLoader.class.getName();

	// Keyed file objects
	private Server m_server;
	private KeyedFile m_stockKf;

	/**
	 * Constructor
 	 */
	public StockLoader(Context context) {
		super(context);
		m_server = null;
		m_stockKf = null;
	}

	/**
	 * Close the server and all the open files
	 */
	private void close()
	{
		if (m_server != null) {
			try {
				// Close the server and everything that hangs
				m_server.close();
			}
			catch (IOException e) {
				Log.e(TAG, "StockLoader closing error: " + e.toString(), e);
			}
			m_server = null;
			m_stockKf = null;
			m_stockLoaderResponse = null;
		}
	}

	/**
	 * Open the keyed file and the server
	 */
	private void open() throws InterruptedException, IOException, KfException
	{
		// Open Sculptor server connection
		m_server = new Server(SERVER_NAME);

		// Open a database
		Database database = m_server.openDB("StockDb", Database.Type.SCULPTOR,
			EnumSet.noneOf(Database.Flag.class), "", 5, STOCK_LOG);

		// Open the keyed file and set the batchRead mode ON
		m_stockKf = database.openTable(STOCK_FILENAME, KeyedFile.Mode.READ, EnumSet.noneOf(KeyedFile.Option.class));
		m_stockKf.batchReadOn();
	}
	/**
	 * Get the remote server stock and return the Product list
	 *
	 * @throws IOException
	 *             I/O exception while receiving data from server.
	 * @throws KfException
	 *             Sculptor Keyed file error
	 */
	private List<Product> loadStock() throws IOException, KfException, OperationCanceledException
	{
		// The list to fill and return
		ArrayList<Product> list = new ArrayList<Product>();

		// Get the fields
		Field itemCode = m_stockKf.getField("ItemCode");
		Field stockLevel = m_stockKf.getField("StockLevel");
		Field reorderLevel = m_stockKf.getField("ReorderLevel");
		Field eOQ = m_stockKf.getField("EOQ");
		Field mOQ = m_stockKf.getField("MOQ");
		Field costPrice = m_stockKf.getField("CostPrice");
		Field salePrice = m_stockKf.getField("SalePrice");
		Field unitCode = m_stockKf.getField("UnitCode");
		Field leadTime = m_stockKf.getField("LeadTime");
		Field nextDelDue = m_stockKf.getField("NextDelDue");
		Field spare = m_stockKf.getField("Spare");
		Field description = m_stockKf.getField("Description");

		m_stockKf.rewind();
		try {
			while (true) {
				// Get next stock product
				m_stockKf.nextu();

				// Create the product with the information received
				Product stock = new Product(itemCode.getString(),
					(int)stockLevel.getLong(),
					(int)reorderLevel.getLong(),
					(int)eOQ.getLong(),
					(int)mOQ.getLong(),
					(int)costPrice.getLong(),
					(int)salePrice.getLong(),
					unitCode.getString(),
					(short)leadTime.getLong(),
					nextDelDue.getString(),
					description.getString()
					);

				// Add the stock product to the list
				list.add(stock);

				// Check if cancelled
				if (isReset())
					throw new OperationCanceledException();
			}
		}
		catch (KfException e) {
			// NSR means no more products
			if (e.getKfStat() != KeyedFile.KfStat.NSR)
				throw e;
		}
		return list;
	}


	/**
	 * Called on a worker thread to perform the actual load and to return
	 * the result of the load operation.
	 * <p/>
	 * Implementations should not deliver the result directly, but should return them
	 * from this method, which will eventually end up calling {@link #deliverResult} on
	 * the UI thread.  If implementations need to process the results on the UI thread
	 * they may override {@link #deliverResult} and do so there.
	 * <p/>
	 * To support cancellation, this method should periodically check the value of
	 * {@link #isLoadInBackgroundCanceled} and terminate when it returns true.
	 * Subclasses may also override {@link #cancelLoadInBackground} to interrupt the load
	 * directly instead of polling {@link #isLoadInBackgroundCanceled}.
	 * <p/>
	 * When the load is canceled, this method may either return normally or throw
	 * {@link OperationCanceledException}.  In either case, the {@link #Loader} will
	 * call {@link #onCanceled} to perform post-cancellation cleanup and to dispose of the
	 * result object, if any.
	 *
	 * @return The result of the load operation.
	 * @throws OperationCanceledException if the load is canceled during execution.
	 * @see #isLoadInBackgroundCanceled
	 * @see #cancelLoadInBackground
	 * @see #onCanceled
	 */
	@Override
	public StockLoaderResponse loadInBackground() {
		try {
			// Open remote resources
			open();

			// Get the results and return the response
			return new StockLoaderResponse(loadStock(), null);
		}
		catch (Exception e) {
			// Log the error and return it
			Log.e(TAG, "StockLoader error: " + e.toString(), e);
			return new StockLoaderResponse(null, e);
		}
		finally {
			// Close remote resources
			close();
		}
	}

	/**
	 * Sends the result of the load to the registered listener. Should only be called by subclasses.
	 * <p/>
	 * Must be called from the process's main thread.
	 *
	 * @param data the result of the load
	 */
	@Override
	public void deliverResult(StockLoaderResponse data)
	{
		if (isReset()) {
			// The Loader has been reset; ignore the result.
			return;
		}

		// Hold a reference to the old data so it doesn't get garbage collected.
		// We must protect it until the new data has been delivered.
		StockLoaderResponse oldData = m_stockLoaderResponse;

		// Save the delivered data
		m_stockLoaderResponse = data;

		if (isStarted()) {
			super.deliverResult(data);
		}
	}

	/**
	 * Subclasses must implement this to take care of loading their data,
	 * as per {@link #startLoading()}.  This is not called by clients directly,
	 * but as a result of a call to {@link #startLoading()}.
	 */
	@Override
	protected void onStartLoading()
	{
		if (m_stockLoaderResponse != null) {
			// Deliver any previously loaded data immediately.
			deliverResult(m_stockLoaderResponse);
		}
		// We should be loading only if we have no previous data or the previous data changed
		// (In theory we should have a monitor running...)
		forceLoad();
	}

	/**
	 * Subclasses must implement this to take care of stopping their loader,
	 * as per {@link #stopLoading()}.  This is not called by clients directly,
	 * but as a result of a call to {@link #stopLoading()}.
	 * This will always be called from the process's main thread.
	 */
	@Override
	protected void onStopLoading()
	{
		// The Loader is in a stopped state, so we should attempt to cancel the
		// current load (if there is one).
		cancelLoad();
	}

	/**
	 * Subclasses must implement this to take care of resetting their loader,
	 * as per {@link #reset()}.  This is not called by clients directly,
	 * but as a result of a call to {@link #reset()}.
	 * This will always be called from the process's main thread.
	 */
	@Override
	protected void onReset()
	{
		// Stop the Loader if it is currently running
		onStopLoading();

		// Release last held data
		m_stockLoaderResponse = null;

		// Close resources
		close();
	}

	/**
	 * Called if the task is canceled before it has completed. Gives the class a chance
	 * to clean up post-cancellation and to dispose the result properly.
	 *
	 * @param data The value that was returned by {@link #loadInBackground} or null
	 *             if the task threw {@link OperationCanceledException}.
	 */
	@Override
	public void onCanceled(StockLoaderResponse data)
	{
		super.onCanceled(data);
	}
}
